/***************************************************************************
 begin       : Fri Jul 30 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@aqbanking.de

 ***************************************************************************
 * This file is part of the project "AqBanking".                           *
 * Please see toplevel file COPYING of that project for license details.   *
 ***************************************************************************/

#ifndef AQBANKING_DLG_USERTYPE_PAGEDEFAULT_L_H
#define AQBANKING_DLG_USERTYPE_PAGEDEFAULT_L_H


#include <aqbanking/banking.h>

#include <gwenhywfar/dialog.h>


#ifdef __cplusplus
extern "C" {
#endif


GWEN_DIALOG *AB_UserTypePageDefaultDialog_new(AB_BANKING *ab);


#ifdef __cplusplus
}
#endif




#endif

