/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "imexporter_accountinfo.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_IMEXPORTER_ACCOUNTINFO_IMEXPORTER_ACCOUNTINFO_P_H
#define AB_IMEXPORTER_ACCOUNTINFO_IMEXPORTER_ACCOUNTINFO_P_H

#include "./imexporter_accountinfo.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_IMEXPORTER_ACCOUNTINFO {
  GWEN_LIST_ELEMENT(AB_IMEXPORTER_ACCOUNTINFO)
  int _refCount;
  char *country;
  char *bankCode;
  char *bankName;
  char *accountNumber;
  char *subAccountId;
  char *accountName;
  char *iban;
  char *bic;
  char *owner;
  char *currency;
  char *description;
  int accountType;
  uint32_t accountId;
  AB_BALANCE_LIST *balanceList;
  AB_TRANSACTION_LIST *transactionList;
  AB_DOCUMENT_LIST *eStatementList;
};

/** 
 * Find an account info object in a list of account info objects. 
 * @param a account info object with which to start searching 
 * @param uniqueId unique account id (use 0 as wildcard) 
 * @param country ISO code of the country (wildcards and jokers allowed) 
 * @param bankId bank code to search for (wildcards and jokers allowed) 
 * @param accountNumber account number to search for (wildcards and jokers allowed) 
 * @param subAccountId sub account number (wildcards and jokers allowed) 
 * @param iban IBAN to search for (wildcards and jokers allowed) 
 * @param currency currecny the account uses (wildcards and jokers allowed) 
 * @param ty account type (use AB_AccountType_Unknown as wildcard) 
 */ 
 AB_IMEXPORTER_ACCOUNTINFO *AB_ImExporterAccountInfo_List__FindInternal(AB_IMEXPORTER_ACCOUNTINFO *a, uint32_t uniqueId, const char *country, const char *bankId, const char *accountNumber, const char *subAccountId, const char *iban, const char *currency, int ty);
#ifdef __cplusplus
}
#endif

#endif

