/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "transaction.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_TRANSACTION_TRANSACTION_P_H
#define AB_TRANSACTION_TRANSACTION_P_H

#include "./transaction.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_TRANSACTION {
  GWEN_LIST_ELEMENT(AB_TRANSACTION)
  int _refCount;
  AB_TRANSACTION_TYPE type;
  AB_TRANSACTION_SUBTYPE subType;
  AB_TRANSACTION_COMMAND command;
  AB_TRANSACTION_STATUS status;
  uint32_t uniqueAccountId;
  AB_TRANSACTION_ACK acknowledge;
  uint32_t uniqueId;
  uint32_t refUniqueId;
  uint32_t idForApplication;
  char *stringIdForApplication;
  uint32_t sessionId;
  uint32_t groupId;
  char *fiId;
  char *localIban;
  char *localBic;
  char *localCountry;
  char *localBankCode;
  char *localBranchId;
  char *localAccountNumber;
  char *localSuffix;
  char *localName;
  char *remoteCountry;
  char *remoteBankCode;
  char *remoteBranchId;
  char *remoteAccountNumber;
  char *remoteSuffix;
  char *remoteIban;
  char *remoteBic;
  char *remoteName;
  GWEN_DATE *date;
  GWEN_DATE *valutaDate;
  AB_VALUE *value;
  AB_VALUE *fees;
  AB_VALUE *taxes;
  int transactionCode;
  char *transactionText;
  char *transactionKey;
  int textKey;
  char *primanota;
  char *purpose;
  char *category;
  char *customerReference;
  char *bankReference;
  char *endToEndReference;
  char *ultimateCreditor;
  char *ultimateDebtor;
  char *creditorSchemeId;
  char *originatorId;
  char *mandateId;
  GWEN_DATE *mandateDate;
  char *mandateDebitorName;
  char *originalCreditorSchemeId;
  char *originalMandateId;
  char *originalCreditorName;
  AB_TRANSACTION_SEQUENCE sequence;
  AB_TRANSACTION_CHARGE charge;
  char *remoteAddrStreet;
  char *remoteAddrZipcode;
  char *remoteAddrCity;
  char *remoteAddrPhone;
  AB_TRANSACTION_PERIOD period;
  uint32_t cycle;
  uint32_t executionDay;
  GWEN_DATE *firstDate;
  GWEN_DATE *lastDate;
  GWEN_DATE *nextDate;
  char *unitId;
  char *unitIdNameSpace;
  char *tickerSymbol;
  AB_VALUE *units;
  AB_VALUE *unitPriceValue;
  GWEN_DATE *unitPriceDate;
  AB_VALUE *commissionValue;
  uint32_t estatementNumber;
  uint32_t estatementMaxEntries;
  AB_TRANSACTION_VOPRESULT vopResult;
  char *memo;
  char *hash; /* volatile */
};

#ifdef __cplusplus
}
#endif

#endif

