/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "security.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_SECURITY_SECURITY_P_H
#define AB_SECURITY_SECURITY_P_H

#include "./security.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_SECURITY {
  GWEN_LIST_ELEMENT(AB_SECURITY)
  int _refCount;
  char *name;
  char *uniqueId;
  char *nameSpace;
  char *tickerSymbol;
  AB_VALUE *units;
  AB_VALUE *unitPriceValue;
  GWEN_TIME *unitPriceDate;
};

#ifdef __cplusplus
}
#endif

#endif

