/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "providerqueue.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_PROVIDERQUEUE_PROVIDERQUEUE_P_H
#define AB_PROVIDERQUEUE_PROVIDERQUEUE_P_H

#include "./providerqueue.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_PROVIDERQUEUE {
  GWEN_INHERIT_ELEMENT(AB_PROVIDERQUEUE)
  GWEN_LIST_ELEMENT(AB_PROVIDERQUEUE)
  int _refCount;
  char *providerName;
  AB_PROVIDER *provider; /* volatile */
  AB_ACCOUNTQUEUE_LIST *accountQueueList; /* volatile */
};

#ifdef __cplusplus
}
#endif

#endif

