/***************************************************************************
    begin       : Sat May 08 2010
    copyright   : (C) 2022 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifndef AQPAYPAL_PROVIDER_P_H
#define AQPAYPAL_PROVIDER_P_H

#include "provider_l.h"

#include <aqbanking/backendsupport/queue.h>
#include <aqbanking/backendsupport/account_p.h>



typedef struct APY_PROVIDER APY_PROVIDER;
struct APY_PROVIDER {
  int dummy;
};





#endif


