/***************************************************************************
    begin       : Thu 21-07-2005
    copyright   : (C) 2005 by Peter de Vrijer
    copyright   : (C) 2018 Martin Preuss
    email       : pdevrijer@home.nl

 ***************************************************************************
 *    Please see the file COPYING in this directory for license details    *
 ***************************************************************************/

#ifndef AQBANKING_IMEX_ERI2_H
#define AQBANKING_IMEX_ERI2_H

#include <aqbanking/backendsupport/imexporter_be.h>
#include <aqbanking/banking.h>


AB_IMEXPORTER *AB_ImExporterERI2_new(AB_BANKING *ab);



#endif /* AQBANKING_IMEX_ERI2_H */
