/***************************************************************************
    begin       : Mon Mar 01 2004
    copyright   : (C) 2018 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifndef AH_JOBSEPACOR1DATEDMULTICREATE_L_H
#define AH_JOBSEPACOR1DATEDMULTICREATE_L_H


#include "accountjob_l.h"


AH_JOB *AH_Job_SepaCor1DebitDatedMultiCreate_new(AB_PROVIDER *pro, AB_USER *u, AB_ACCOUNT *account);


#endif /* AH_JOBSEPACOR1DATEDMULTICREATE_L_H */


