/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "bankinfo.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_BANKINFO_BANKINFO_P_H
#define AB_BANKINFO_BANKINFO_P_H

#include "./bankinfo.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_BANKINFO {
  GWEN_LIST_ELEMENT(AB_BANKINFO)
  int _refCount;
  char *country;
  char *branchId;
  char *bankId;
  char *bic;
  char *bankName;
  char *location;
  char *street;
  char *zipcode;
  char *city;
  char *region;
  char *phone;
  char *fax;
  char *email;
  char *website;
  AB_BANKINFO_SERVICE_LIST *services;
};

#ifdef __cplusplus
}
#endif

#endif

