/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "message.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_MESSAGE_MESSAGE_P_H
#define AB_MESSAGE_MESSAGE_P_H

#include "./message.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_MESSAGE {
  GWEN_LIST_ELEMENT(AB_MESSAGE)
  int _refCount;
  AB_MESSAGE_SOURCE source;
  uint32_t userId;
  uint32_t accountId;
  char *subject;
  char *text;
  GWEN_TIME *dateReceived;
};

#ifdef __cplusplus
}
#endif

#endif

