/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "accountqueue.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_ACCOUNTQUEUE_ACCOUNTQUEUE_P_H
#define AB_ACCOUNTQUEUE_ACCOUNTQUEUE_P_H

#include "./accountqueue.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_ACCOUNTQUEUE {
  GWEN_INHERIT_ELEMENT(AB_ACCOUNTQUEUE)
  GWEN_LIST_ELEMENT(AB_ACCOUNTQUEUE)
  int _refCount;
  uint32_t accountId;
  AB_ACCOUNT *account; /* volatile */
  AB_ACCOUNT_SPEC *accountSpec; /* volatile */
  AB_TRANSACTION_LIST2 *transactionList; /* volatile */
  AB_JOBQUEUE_LIST *jobQueueList; /* volatile */
};

#ifdef __cplusplus
}
#endif

#endif

