/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "refaccount.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_REFERENCE_ACCOUNT_REFACCOUNT_P_H
#define AB_REFERENCE_ACCOUNT_REFACCOUNT_P_H

#include "./refaccount.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_REFERENCE_ACCOUNT {
  GWEN_LIST_ELEMENT(AB_REFERENCE_ACCOUNT)
  int _refCount;
  char *iban;
  char *bic;
  char *accountNumber;
  char *subAccountNumber;
  char *country;
  char *bankCode;
  char *ownerName;
  char *ownerName2;
  char *accountName;
  uint8_t accountType;
};

/** 
 * Find an account info object in a list of account info objects. 
 * @param a account info object with which to start searching 
 * @param iban IBAN to search for (wildcards and jokers allowed) 
 * @param bic BIC to search for (wildcards and jokers allowed) 
 * @param accountNumber account number to search for (wildcards and jokers allowed) 
 * @param subAccountNumber sub account number (wildcards and jokers allowed) 
 * @param country ISO code of the country (wildcards and jokers allowed) 
 * @param bankCode bank code to search for (wildcards and jokers allowed) 
 * @param ownerName owner name to search for (wildcards and jokers allowed) 
 * @param accountName account name to search for (wildcards and jokers allowed) 
 */ 
 AB_REFERENCE_ACCOUNT *AB_ReferenceAccount_List__FindInternal(AB_REFERENCE_ACCOUNT *a, const char *iban, const char *bic, const char *accountNumber, const char *subAccountNumber, const char *country, const char *bankCode, const char *ownerName, const char *accountName);
#ifdef __cplusplus
}
#endif

#endif

