/***************************************************************************
    begin       : Wed Jan 12 2022
    copyright   : (C) 2025 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifndef AH_JOBQUEUE_ACCOUNT_H
#define AH_JOBQUEUE_ACCOUNT_H

#include "aqhbci/joblayer/jobqueue_l.h"


void AH_JobQueue_ReadAccounts(AH_JOBQUEUE *jq, GWEN_DB_NODE *dbResponses);
void AH_JobQueue_ReadSepaAccounts(AH_JOBQUEUE *jq, GWEN_DB_NODE *dbResponses);



#endif




