/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "bankinfoservice.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_BANKINFO_SERVICE_BANKINFOSERVICE_P_H
#define AB_BANKINFO_SERVICE_BANKINFOSERVICE_P_H

#include "./bankinfoservice.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_BANKINFO_SERVICE {
  GWEN_LIST_ELEMENT(AB_BANKINFO_SERVICE)
  int _refCount;
  char *type;
  char *address;
  char *suffix;
  char *pversion;
  char *hversion;
  char *mode;
  char *aux1;
  char *aux2;
  char *aux3;
  char *aux4;
  uint32_t userFlags;
};

#ifdef __cplusplus
}
#endif

#endif

