/***************************************************************************
    begin       : Mon Mar 01 2004
    copyright   : (C) 2023 by Rico Rommel
    email       : rico@bierrommel.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifndef AG_PROVIDER_P_H
#define AG_PROVIDER_P_H

#include "provider.h"
#include <aqbanking/banking.h>
#include <aqbanking/backendsupport/providerqueue.h>
#include <aqbanking/backendsupport/user.h>
#include <aqbanking/backendsupport/account.h>



struct AG_PROVIDER {
  int dummy;
};



#endif

