/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "document.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_DOCUMENT_DOCUMENT_P_H
#define AB_DOCUMENT_DOCUMENT_P_H

#include "./document.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_DOCUMENT {
  GWEN_LIST_ELEMENT(AB_DOCUMENT)
  int _refCount;
  char *id;
  uint32_t ownerId;
  char *mimeType;
  char *filePath;
  GWEN_BINDATA data;
  GWEN_BINDATA acknowledgeCode;
};

#ifdef __cplusplus
}
#endif

#endif

