/***************************************************************************
    begin       : Tue Mar 31 2009
    copyright   : (C) 2018 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifndef AQHBCI_IMEX_CTXFILE_H
#define AQHBCI_IMEX_CTXFILE_H


#include <gwenhywfar/dbio.h>
#include <aqbanking/backendsupport/imexporter_be.h>


AB_IMEXPORTER *AB_ImExporterCtxFile_new(AB_BANKING *ab);


#endif /* AQHBCI_IMEX_CTXFILE_H */
