/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "imexporter_context.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_IMEXPORTER_CONTEXT_IMEXPORTER_CONTEXT_P_H
#define AB_IMEXPORTER_CONTEXT_IMEXPORTER_CONTEXT_P_H

#include "./imexporter_context.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_IMEXPORTER_CONTEXT {
  GWEN_LIST_ELEMENT(AB_IMEXPORTER_CONTEXT)
  int _refCount;
  AB_IMEXPORTER_ACCOUNTINFO_LIST *accountInfoList;
  AB_SECURITY_LIST *securityList;
  AB_MESSAGE_LIST *messageList;
};

#ifdef __cplusplus
}
#endif

#endif

