/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "transactionlimits.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_TRANSACTION_LIMITS_TRANSACTIONLIMITS_P_H
#define AB_TRANSACTION_LIMITS_TRANSACTIONLIMITS_P_H

#include "./transactionlimits.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_TRANSACTION_LIMITS {
  GWEN_LIST_ELEMENT(AB_TRANSACTION_LIMITS)
  int _refCount;
  int command;
  int maxLenLocalName;
  int minLenLocalName;
  int maxLenRemoteName;
  int minLenRemoteName;
  int maxLenCustomerReference;
  int minLenCustomerReference;
  int maxLenBankReference;
  int minLenBankReference;
  int maxLenPurpose;
  int minLenPurpose;
  int maxLinesPurpose;
  int minLinesPurpose;
  int needDate;
  int minValueSetupTime;
  int maxValueSetupTime;
  int minValueSetupTimeFirst;
  int maxValueSetupTimeFirst;
  int minValueSetupTimeOnce;
  int maxValueSetupTimeOnce;
  int minValueSetupTimeRecurring;
  int maxValueSetupTimeRecurring;
  int minValueSetupTimeFinal;
  int maxValueSetupTimeFinal;
  uint8_t valuesCycleWeek[64];
  int valuesCycleWeekUsed;
  uint8_t valuesCycleMonth[64];
  int valuesCycleMonthUsed;
  uint8_t valuesExecutionDayWeek[16];
  int valuesExecutionDayWeekUsed;
  uint8_t valuesExecutionDayMonth[64];
  int valuesExecutionDayMonthUsed;
  int allowMonthly;
  int allowWeekly;
  int allowChangeRecipientAccount;
  int allowChangeRecipientName;
  int allowChangeValue;
  int allowChangeTextKey;
  int allowChangePurpose;
  int allowChangeFirstExecutionDate;
  int allowChangeLastExecutionDate;
  int allowChangeCycle;
  int allowChangePeriod;
  int allowChangeExecutionDay;
};

#ifdef __cplusplus
}
#endif

#endif

