/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "balance.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_BALANCE_BALANCE_P_H
#define AB_BALANCE_BALANCE_P_H

#include "./balance.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_BALANCE {
  GWEN_LIST_ELEMENT(AB_BALANCE)
  int _refCount;
  GWEN_DATE *date;
  AB_VALUE *value;
  AB_BALANCE_TYPE type;
};

#ifdef __cplusplus
}
#endif

#endif

