/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "user.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_USER_USER_P_H
#define AB_USER_USER_P_H

#include "./user.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_USER {
  GWEN_INHERIT_ELEMENT(AB_USER)
  GWEN_LIST_ELEMENT(AB_USER)
  int _refCount;
  uint32_t uniqueId;
  char *backendName;
  char *userName;
  char *userId;
  char *customerId;
  char *country;
  char *bankCode;
  AB_PROVIDER *provider; /* volatile */
  GWEN_DB_NODE *certDb;
  uint32_t lastSessionId;
  AB_USER_READFROMDB_FN readFromDbFn;
  AB_USER_WRITETODB_FN writeToDbFn;
};

/** 
 */ 
 int AB_User__ReadFromDb(AB_USER *a, GWEN_DB_NODE *db);
/** 
 */ 
 int AB_User__WriteToDb(const AB_USER *a, GWEN_DB_NODE *db);
#ifdef __cplusplus
}
#endif

#endif

