/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "paymentgroup.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_IMEXPORTER_XML_PAYMENTGROUP_PAYMENTGROUP_P_H
#define AB_IMEXPORTER_XML_PAYMENTGROUP_PAYMENTGROUP_P_H

#include "./paymentgroup.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_IMEXPORTER_XML_PAYMENTGROUP {
  GWEN_LIST_ELEMENT(AB_IMEXPORTER_XML_PAYMENTGROUP)
  char *id;
  int transactionCount;
  AB_VALUE *controlSum;
  GWEN_DATE *transactionDate;
  int sequence;
  char *creditorSchemeId;
  char *localIban;
  char *localBic;
  char *localName;
  AB_TRANSACTION_LIST2 *transactionList2;
  char *hash; /* volatile */
};

#ifdef __cplusplus
}
#endif

#endif

