/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "queue.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_QUEUE_QUEUE_P_H
#define AB_QUEUE_QUEUE_P_H

#include "./queue.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_QUEUE {
  GWEN_INHERIT_ELEMENT(AB_QUEUE)
  int _refCount;
  AB_USERQUEUE_LIST *userQueueList; /* volatile */
};

#ifdef __cplusplus
}
#endif

#endif

