/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "swiftdescr.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AB_SWIFT_DESCR_SWIFTDESCR_P_H
#define AB_SWIFT_DESCR_SWIFTDESCR_P_H

#include "./swiftdescr.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AB_SWIFT_DESCR {
  GWEN_LIST_ELEMENT(AB_SWIFT_DESCR)
  int _refCount;
  char *family;
  int version1;
  int version2;
  int version3;
  char *alias1;
  char *alias2;
};

/** 
 * Find a SWIFT descriptor in a list by given criteria. 
 * @param a descriptor object with which to start searching 
 * @param wantedFamily descriptor family, e.g. "pain" or "camt" (wildcards and jokers allowed) 
 * @param wantedVersion1 first part of the version to match (0 matches all) 
 * @param wantedVersion2 second part of the version to match (0 matches all) 
 * @param wantedVersion3 third part of the version to match (0 matches all) 
 */ 
 AB_SWIFT_DESCR *AB_SwiftDescr_List__FindInternal(AB_SWIFT_DESCR *d, const char *wantedFamily, int wantedVersion1, int wantedVersion2, int wantedVersion3);
#ifdef __cplusplus
}
#endif

#endif

