/**********************************************************
 * This file has been automatically created by "typemaker2"
 * from the file "tanmethod.xml".
 * Please do not edit this file, all changes will be lost.
 * Better edit the mentioned source file instead.
 **********************************************************/

#ifndef AH_TAN_METHOD_TANMETHOD_P_H
#define AH_TAN_METHOD_TANMETHOD_P_H

#include "./tanmethod.h"


#ifdef __cplusplus
extern "C" {
#endif

struct AH_TAN_METHOD {
  GWEN_INHERIT_ELEMENT(AH_TAN_METHOD)
  GWEN_LIST_ELEMENT(AH_TAN_METHOD)
  int _refCount;
  int function;
  int process;
  char *methodId;
  char *methodName;
  char *zkaTanName;
  char *zkaTanVersion;
  int tanMaxLen;
  char *formatId;
  char *prompt;
  int returnMaxLen;
  int maxActiveLists;
  int multiTanAllowed;
  int timeShiftAllowed;
  int tanListMode;
  int stornoAllowed;
  int needSmsAccount;
  int needLocalAccount;
  int needChallengeClass;
  int needChallengeAmount;
  int challengeIsStructured;
  int initMode;
  int needTanMediumId;
  int maxActiveTanMedia;
  int gvVersion;
};

#ifdef __cplusplus
}
#endif

#endif

